// {{{ Copyright (C) 2009 Nuno David Lopes.-----------------------------------
//
// e-mail: ndl@ptmat.fc.ul.pt
// http://ptmat.fc.ul.pt/~ndl/
//
// ---------------------------------------------------------------------------
// Licensed under the GNU LGPL Version 3.0 or later.
//
// ---------------------------------------------------------------------------
//  main.cpp
//
//  This program provides a demo of the DOLFWAVE library. 
//  Standard Gaussian Wave
//  Initial Conditions given in files "eta_init" and "phi_init"
//  --------------------------------------------------------------------------
//  Using Xd3d for pr post-processing.
//
// }}}------------------------------------------------------------------------
// {{{ Includes --------------------------------------------------------------
#include <dolfwave.h>
// }}} -----------------------------------------------------------------------
using namespace dolfin;
using namespace dolfin::dolfwave;
int main()
{
  // {{{  Clear and maintimer.start()
  Dolfwave dw(
              2000 /*MaxSteps*/,
              0.001 /*TimeStep*/, 
              50 /*WriteGap*/,
              "useless" /*MeshName*/,
              1. /*PcEpsilon*/,
              "Zhao" /*FormName*/,
              "uBLAS" /*AlgBackEND*/,
              "xd3d_xyz" /*Plotter*/,
              "output/eta.raw" /*EtaFile*/,
              "output/phi.raw"/*PhiFile*/,
              "output/phyphi.raw" /*PhyPhiFile*/,
              "output/horvel.raw" /*HorVelFile*/
              ); 
  dw.TimeAdd(0.03); //Starting point of Time 
  /* 
     Just to remember that time should be updated 
     This is important when time-dependent Dirichlet BC 
     are applied (which is not the case) 
  */
  // }}}
  // {{{Mesh, boundary definition, Function Spaces
  Rectangle mesh(0,0,6,6,35,35,Rectangle::left);
  mesh.order();
  Constant g(9.81);//Depth
  Constant depth(0.45);//Depth
  Constant sponge(0.0);//Sponge Layer
  Constant src(0.0);//Source Function
  dw.FunctionSpaceInit(mesh);
  // }}}
  // {{{ Functions, Vectors and Matrices ----------------------
  dw.BilinearFormInit(depth);
  dw. MatricesAssemble( );  
  dw.FunctionsInit( );// Unknown functions initialization 
  dw.InitialCondition("eta_init","phi_init"); //Load Int.Cond. from files eta_init and phi_init (raw format)
  dw.Plot(true,false,false);
  // }}}-----------------------------------------------------------------
  dw.LinearFormsInit(depth,g,
                     sponge /*sp_eta*/,sponge /*sp_lap_eta*/,
                     sponge /*sp_phi*/,sponge/*sp_lap_phi */,
                     src);
  dw.VectorInit( ); //Auxiliar vector initialization
  // }}}---------------------------------------------------------
  dw.LinearSolver( ); //Seting the linear Solver (limited yet)
  dw.RKSolve( ); // Runge-Kutta explicit fist 3 steps
  dw.PCInit( );  //PC Initialization 
  for(dolfin::uint i=4; i<dw.MaxSteps+1;i++)
    {
      dw.PCSolve( );     
      //  Plot information------------------------
      if (!(i%dw.WriteGap))
        dw.Plot(true,true,false,false);
      message("Step=%d \t Simulated time= %.4f",i,dw.Time);
    }
  
  return (EXIT_SUCCESS);
}
