// {{{  Copyright (C) 2009 Nuno David Lopes.-----------------------------------
//
// e-mail: ndl@ptmat.fc.ul.pt
// http://ptmat.fc.ul.pt/~ndl/
// ---------------------------------------------------------------------------
// Licensed under the GNU LGPL Version 3.0 or later.
// ---------------------------------------------------------------------------
//  main.cpp
//
//  This program provides a demo of the DOLFWAVE library. 
//  Wave generation by a moving bottom: Modified 2nd-order equations
// --------------------------------------------------------------------------
// Using Viper  pr post-processor and Xd3d with the files output.
// }}}------------------------------------------------------------------------
#include <dolfwave.h>
using namespace dolfin;
using namespace dolfin::dolfwave;
class Depth : public Function
{
 public:
  Depth (const double& t): t(t)
  {}
private:
  void eval(double* values, const double* x) const
  {
    double time=t;
    double endtime=2.0;
    double D=0.04575;
    double xS=8.0;
    double omega=DOLFIN_PI;
    double gamma=3.0;
    //(Time derivative of Depth)(x-1)
    if (time<endtime)
      values[0]=0.4572-D*sin(omega*time)*
        exp(-gamma*((x[0]-xS)*(x[0]-xS)+(x[1]-xS)*(x[1]-xS))) ;
    else values[0]=0.4572; 
  }
  const double& t;
};
class Source : public Function
{
 public:
  Source (const double& t): t(t)
  {}
private:
  void eval(double* values, const double* x) const
  {
    double time=t;
    double endtime=2.0;
    double D=0.04572;
    double xS=8.0;
    double omega=DOLFIN_PI;
    double gamma=3.0;
    if (time<endtime)
      values[0]=D*omega*cos(omega*time)*
        exp(-gamma*((x[0]-xS)*(x[0]-xS)+(x[1]-xS)*(x[1]-xS))) ;
    else values[0]=0.0;
  }
  const double& t;
};
int main()
{
  Dolfwave dw(
              10000  /*MaxSteps*/,
              0.001 /*TimeStep*/,
              100 /*WriteGap*/,
              "../../meshes/square/square.xml.gz" /*MeshName*/,
              0.0 /*PcEpsilon*/,
              "LPT2z0" /*FormName*/,
              "uBLAS"/* AlgBackEND*/ ,
              "xd3d_xyz" /*Plotter*/,
              "output/eta.raw" /*EtaFile*/,
              "output/phi.raw"/*PhiFile*/,
              "output/phyphi.raw" /*PhyPhiFile*/,
              "output/horvel.raw" /*HorVelFile*/
              ); 
  Mesh mesh(dw.MeshName);
  mesh.order();
  Constant g(9.81); //Gravity acceleration
  Depth depth(dw.Time);//time variable depth
  Constant D(0.4572);//Constant depth term (From LPT2z form)
  Constant sponge(0.0);//Sponge Layer
  Source src(dw.Time);//Source Function
  dw.FunctionSpaceInit(mesh);//Function Spaces
  dw.BilinearFormInit(D);//Bilinear form initialization
  dw.FunctionsInit();  //Unknown functions
  dw.LinearFormsInit(depth,g,sponge,src);//Linear form initialization
  dw.VectorInit();//Auxiliar vectors initialization
  dw.MatricesAssemble( );//Matrices Assemble
  dw.LinearSolver();//Linear solver (limited yet) 
  dw.RKSolve( ); //first 3 steps
  dw.PCInit(); //P.Corrector initialization
  for(dolfin::uint i=4; i<dw.MaxSteps+1;i++)
    {dw.PCSolve( );//PC step    
      if (!(i%dw.WriteGap))
        dw.Plot(true,true,false,false); //ploting and saving
      message("Step=%d \t Simulated time= %.4f",i,dw.Time);}
  return (EXIT_SUCCESS); 
}
