// {{{  Copyright (C) 2009 Nuno David Lopes.-----------------------------------
//
// e-mail: ndl@ptmat.fc.ul.pt
// http://ptmat.fc.ul.pt/~ndl/
// ---------------------------------------------------------------------------
// Licensed under the GNU LGPL Version 3.0 or later.
// ---------------------------------------------------------------------------
//  main.cpp
//  This program provides a demo of the DOLFWAVE library. 
//  Wave generation by Dirichlet boundary conditions
// --------------------------------------------------------------------------
// Using Viper  pr post-processor and Xd3d with the files output.
// }}}------------------------------------------------------------------------
#include <dolfwave.h>
using namespace dolfin;
using namespace dolfin::dolfwave;
// Dirichlet Boundary conditions
class DirichletBoundary : public SubDomain
{
  bool inside(const double* x, bool on_boundary) const
  {
    return (x[0]<DOLFIN_EPS) && on_boundary;    
  }
};
class DBC : public Function
{
public:
  DBC(const double& t) : t(t)
  {}
private:
  void eval(double* values,const double* x) const
  {
    double time=t;
    double a=0.01;
    double omega=3.1142485275;
    double b=0.044861357;
    double k=1.6845;
    values[0]= -a*sin(omega*time);
    values[1]=-(b/k)*cos(omega*time)+b/k;
  }
  const double& t;
};
int main()
{ // Dolfwave Initialization
  Dolfwave dw(10000  /*MaxSteps*/,
              0.001 /*TimeStep*/,
              500 /*WriteGap*/,
              "../../meshes/pillar/rectpillar.xml.gz" /*MeshName*/,
              1. /*PcEpsilon*/,
              "LPT2z0" /*FormName*/,
              "uBLAS"/* AlgBackEND*/ ,
              "viper" /*Plotter*/,
              "output/eta.raw" /*EtaFile*/,
              "output/phi.raw"/*PhiFile*/,
              "output/phyphi.raw" /*PhyPhiFile*/,
              "output/horvel.raw" /*HorVelFile*/); 
  Mesh mesh(dw.MeshName);//Mesh definition
  mesh.order();
  Constant g(9.81);
  Constant depth(0.4572);//Depth
  Constant zero(0.0);//Sponge Layer
  DirichletBoundary dirichletboundary;
  DBC bc(dw.Time);
  dw.FunctionSpaceInit(mesh);
  dw.BilinearFormInit(depth);
  dw.FunctionsInit();// For zero initial conditions just call FunctionsInit 
  dw.LinearFormsInit(depth,g,zero,zero);
  dw.VectorInit( );
  dw.DolfwaveDBC(bc,dirichletboundary);//Dirichlet B.C. definition  
  dw.MatricesAssemble( );  
  dw.LinearSolver();//Linear solver (limited yet) 
  dw.RKSolve(); //first 3 steps
  dw.PCInit(); //P.Corrector initialization
  for(dolfin::uint i=4; i<dw.MaxSteps+1;i++)
    {dw.PCSolve( );//PC step    
      if (!(i%dw.WriteGap))
        dw.Plot(true,false,false); //ploting and saving
      message("Step=%d \t Simulated time= %.4f",i,dw.Time);}
  
  return (EXIT_SUCCESS);
}
