// {{{ Copyright (C) 2009 Nuno David Lopes.-----------------------------------
//
// e-mail: ndl@ptmat.fc.ul.pt
// http://ptmat.fc.ul.pt/~ndl/
//
// ---------------------------------------------------------------------------
// Licensed under the GNU LGPL Version 3.0 or later.
//
// ---------------------------------------------------------------------------
//  main.cpp
//
//  This program provides a demo of the DOLFWAVE library. 
//  Standard Gaussian Wave
//  --------------------------------------------------------------------------
//  Using Viper for preview.
//
// }}}------------------------------------------------------------------------
// {{{ Includes --------------------------------------------------------------
#include <dolfwave.h>
// }}} -----------------------------------------------------------------------
using namespace dolfin;
using namespace dolfin::dolfwave;
// {{{--------------------------------------------------------------------
class ElevationInit : public Function
{
  void eval(double* values,const double* x) const
  {
    values[0]=0.045*exp(-2.*(sqr(x[0]-3.0)+sqr(x[1]-3.0))); 
  }
};
// }}}--------------------------------------------------------------------
int main()
{
  // {{{ Options, Initialization, Mesh and User functions
  Dolfwave dw(
              5000 /*MaxSteps*/,
              0.001 /*TimeStep*/,               
              100 /*WriteGap*/,
              "useless" /*MeshName*/,
              1.0e-8 /*PcEpsilon*/,
              "Zhao" /*FormName*/,
              "uBLAS" /*AlgBackEND*/,
              "viper" /*Plotter*/,
              "output/eta.raw" /*EtaFile*/,
              "output/phi.raw"/*PhiFile*/,
              "output/phyphi.raw" /*PhyPhiFile*/,
              "output/horvel.raw" /*HorVelFile*/
              ); 
  Rectangle mesh(0,0,6,6,35,35,Rectangle::left);
  mesh.order();
  Constant depth(0.45);//Depth
  Constant g(9.81);//Gravity acceleration $m/s^2$
  Constant sponge(0.0);//Sponge Layer
  Constant src(0.0);//Source Function
  Constant  phi_init(0.0);//initial condition for potential
  ElevationInit eta_init;//initial condition for elevation
  // }}} -----------------------------------------------------------
  // {{{ Functions, Vectors and Matrices ----------------------
  dw.FunctionSpaceInit(mesh); //Function Spaces Init
  dw.BilinearFormInit(depth);//BilinearForm a initialization
  dw.MatricesAssemble( ); //Matrices M and Mf initialization  
  dw.FunctionsInit( );// Solutions Functions (potential,elevation) 
  dw.LinearFormsInit(depth,g,
                     sponge /*sp_eta*/,sponge /*sp_lap_eta*/,
                     sponge /*sp_phi*/,sponge/*sp_lap_phi */,src); 
  dw.InitialCondition(eta_init,phi_init); //Initial Conditions L2 projection
  dw.VectorInit( );//Auxiliar vectors Initialization
  dw.LinearSolver( ); //Solver type
  // }}} -----------------------------------------------------------------
  dw.RKSolve( );//Runge-Kutta for the initialization of the 3 initial steps
  // {{{Loop with Predictor-Corrector method------------------   
  dw.PCInit( );
  for(dolfin::uint i=4; i<dw.MaxSteps+1;i++)
    {
      dw.PCSolve( );
      //  Plot information------------------------
      if (!(i%dw.WriteGap))
        {
          dw.Plot(true /*eta Plotter*/,false/*phi Plotter*/, 
                  false /*SaveEta*/,false /*SavePhi*/);
          dw.PointPlot(3.,3.,dw.eta,"p1.te");
          dw.PointPlot(3.,3.,dw.phi,"p1.tp");
        }
      message("Step=%d \t Simulated time= %.4f",i,dw.Time);
    }
  // }}}------------------------------------------------------
  
  return (EXIT_SUCCESS);
}
