from pylab import *

#alpha definition
alpha=0.6220355269

#ranges
x0=arange(0.0,20.,0.1)




# Full Linear Airy (\tau=\nu_1=\nu_2=0)
class AIRY:
    def __init__(self):
        print 'airy class'

    def phase(self,x):
        return sqrt(tanh(x)/x)

    def group(self,x):
        return self.phase(x)*(1+2*x/sinh(2*x))/2.

    def shoal(self,x):
        return 2*x*(sinh(2*x)+x*(1-cosh(2*x)))/((2*x+sinh(2*x))**2)

# Zhao Cheng & Teng Second Order Model (\tau=\nu_1=\nu_2=0)
#generic phase velocity
def phase(a,b,c,d,x):
    return sqrt((1.-a*x**2+b*x**4)/(1-c*x**2+d*x**4))

#generic group velocity
def group(a,b,c,d,x):
    return sqrt((1.-a*x**2+b*x**4)/(1-c*x**2+d*x**4))+\
           ((4.*b*x**4-2.*a*x**2)/(1-c*x**2+d*x**4)-\
            (4.*d*x**4-2.*c*x**2)*(1.-a*x**2+b*x**4)/\
            (1-c*x**2+d*x**4)**2)/\
            (2.*sqrt((1.-a*x**2+b*x**4)/(1-c*x**2+d*x**4)))

class ZCT:
    def __init__(self):
        print 'zct class'
    #zhao cheng teng model
    def phase(self,x):
        return sqrt((1.+x**2/15.)/(1.+2.*x**2/5))

    def group(self,x):
        a=-1./15.
        b=0.
        c=-2./5.
        d=0.
        return group(a,b,c,d,x)
    

    def c1(self,x):
        return 2.*(1.+2./15.*x**2+2./75.*x**4)
    
    def c2(self,x):
        return (1.+2./5.*x**2+2./15.*x**4)
    
    def c3(self,x):
        return (1.-2./5.*x**2+2./25.*x**4) 
    
    def c4(self,x):
        return (1.+1./5.*x**2+2./75.*x**4)/\
               (2.*(1.+1./15.*x**2+1./75.*x**4))
    
    def shoal(self,x):
        return (self.c3(x)-self.c2(x)*self.c4(x))/self.c1(x)




#LPT 6^{th}-order model (\tau=\nu_1=\nu_2=0)
# Ti coefficients
def T(Ca,Cb,x):
    return 1.-Ca*x**2+Cb*x**4

class LPTt16o:
    def __init__(self):
        print 'LPT T-I 6-order class'
        
        
    def phase(self,x):
        C1=alpha**2/2.-alpha+1./3.
        C2=5./24.*alpha**4-5./6.*alpha**3+7./6.*alpha**2-2./3.*alpha+2./15.
        C3=0.0
        C4=0.0
        C6=alpha**2/2.-alpha
        C7=5./24.*alpha**4-5./6.*alpha**3+alpha**2-alpha/3.
        print C1
        print C2
        print C3
        print C4
        print C6
        print C7
        return sqrt(T(C1,C2,x)/(T(C3,C4,x)*T(C6,C7,x)))

class LPTt14o:
    def __init__(self):
        print 'LPT T-I 4-order class'
        
    def phase(self,x):
        C1=alpha**2/2.-alpha+1./3.
        C2=0.0
        C3=0.0
        C4=0.0
        C6=alpha**2/2.-alpha
        C7=-alpha**2/6.+alpha/3.-2./15.
        print C1
        print C2
        print C3
        print C4
        print C6
        print C7
        return sqrt(T(C1,C2,x)/(T(C3,C4,x)*T(C6,C7,x)))
    
    



################## PLOT COMMANDS #######################
airy=AIRY()
zct=ZCT()
lptt16o=LPTt16o()
lptt14o=LPTt14o()

figure(1)
subplots_adjust(left=0.10,right=0.90,top=0.90, bottom=0.25)
subplot(111)

xlabel('kH')
ylabel('Phase velocity $C/\sqrt{gH}$')

plot(x0,airy.phase(x0),label='airy')
plot(x0,zct.phase(x0),label='zct')
plot(x0,lptt16o.phase(x0),label='T-I 6-o')
plot(x0,lptt14o.phase(x0),label='T-I 4-o')
legend(loc=3)

show()
