from pylab import *
from matplotlib.widgets import Slider
from boussdispclass import *

## change to 1 if want to save the pictures
save=1

#initial guess for alpha
#6-order z=alpha model
a0=0.553365384615
#4-order z=alpha model
a1=0.560817307692
a2=0.168269230769

#ranges
x0=arange(0.0,1.5,0.01)
x01=arange(0.0,1.0,0.01)
x1=arange(0.0,10.0,0.01)
x2=arange(0.0,20,0.01)

#Class instances from shoaling.py
airy=AIRY()
z06=Z06()
z04=Z04()
za6=ZA6()
za4=ZA4()
za0a14=ZA0A14()
zbeta4=ZBETA4()
zct=ZCT()
nwogu1=NWG1()
nwogu2=NWG2()

figure(1)
subplots_adjust(left=0.05,right=0.99,top=0.99, bottom=0.25)
subplot(321)

xlabel('kH')
ylabel('Phase velocity $C/\sqrt{gH}$')

plot(x0,airy.phase(x0),label='airy')
plot(x0,zct.phase(x0),label='zct')
plot(x0,z06.phase(x0),label='$z=0, 6^{th}$ order ')
plot(x0,z04.phase(x0),label='$z=0, 4^{th}$ order ')
#plot(x0,my3(x0),label='$z=0, 4^{th}$ order 2')
legend(loc=3)

subplot(322)
xlabel('kH')
plot(x1,airy.phase(x1),label='airy')
#plot(x,my3,label='my3')
plot(x1,zct.phase(x1),label='zct')
# #plot(x,nwogu1.phase(x1),label='nw1')
plot(x1,nwogu2.phase(x1),label='nw2')
l6,= plot(x1,za6.phase(a0,x1),label='$z=-a h, 6^{th}$ order' )
l4, = plot(x1,za4.phase(a1,x1),label='$z=-a h, 4^{th}$ order' )
legend()


subplot(323)
xlabel('kH')
ylabel('Group velocity $C_g/\sqrt{gH}$')

plot(x0,airy.group(x0),label='airy')
plot(x0,zct.group(x0),label='zct')
plot(x0,z06.group(x0),label='$z=0, 6^{th}$ order ')
plot(x0,z04.group(x0),label='$z=0, 4^{th}$ order ')
#plot(x0,my3(x0),label='$z=0, 4^{th}$ order 2')
#legend(loc=2)

subplot(324)
xlabel('kH')


plot(x1,airy.group(x1),label='airy')
#plot(x,my3,label='my3')
plot(x1,zct.group(x1),label='zct')
#plot(x,nwogu1.group(x1),label='nw1')
plot(x1,nwogu2.group(x1),label='nw2')
l6g,= plot(x1,za6.group(a0,x1),label='$z=-a h, 6^{th}$ order' )
l4g,= plot(x1,za4.group(a1,x1),label='$z=-a h, 4^{th}$ order' )
#legend(loc=1)

subplot(325)
xlabel('kH')
ylabel('$\\alpha_s$ Shoaling coefficient')
plot(x01,airy.shoal(x01),label='airy')
plot(x01,zct.shoal(x01),label='zct')
plot(x01,z06.shoal(x01),label='z=0,6-order')
plot(x01,z04.shoal(x01),label='z=0,4-order')
#legend()


subplot(326)
xlabel('kH')
plot(x2,airy.shoal(x2),label='airy')
plot(x2,zct.shoal(x2),label='zct')
plot(x2,nwogu2.shoal(x2),label='nw1')
l6s,= plot(x2,za6.shoal(a0,x2),label='$z=-a h, 6^{th}$ order' )
l4s,= plot(x2,za4.shoal(a1,x2),label='$z=-a h, 4^{th}$ order' )
la0a14s,= plot(x2,za0a14.shoal(a1,a2,x2),label='$z=-a0h0-a1\delta h1, 4^{th}$ order' )
lbetas,= plot(x2,zbeta4.shoal(a1,a2,1.0,x2),label='$z=-a0h0-a1\delta h1 beta, 4^{th}$ order' )
#legend()



##Slider and update data
color6= 'white'
alphamp6  = axes([0.25, 0.15, 0.65, 0.03],axisbg=color6)

color4 = 'lightgoldenrodyellow'
alphamp4  = axes([0.25, 0.1, 0.65, 0.03],axisbg=color4)
alphamp41  = axes([0.25,0.05,0.65, 0.03],axisbg=color4)
alphamp4beta  = axes([0.25,0.00,0.65, 0.03],axisbg=color4)

salph6 = Slider(alphamp6, 'alpha $6^{th}$ order', 0.4, 0.6, valinit=a0)
salph4 = Slider(alphamp4, 'alpha $4^{th}$ order', 0.4, 0.6, valinit=a1)
salph41 = Slider(alphamp41, 'alpha 1 $4^{th}$ order', 0.0, 1.0, valinit=a2)
salph4beta = Slider(alphamp4beta, 'beta alpha 1 $4^{th}$ order', 0.0, 1.0, valinit=1.0)

# Slider definitions

def up6(val):
    amp = salph6.val
    l6.set_ydata(za6.phase(amp,x1))
    l6g.set_ydata(za6.group(amp,x1))
    l6s.set_ydata(za6.shoal(amp,x2))
    draw()

def up4(val):
    amp=salph4.val
    l4.set_ydata(za4.phase(amp,x1))
    l4g.set_ydata(za4.group(amp,x1))
    l4s.set_ydata(za4.shoal(amp,x2))
    la0a14s.set_ydata(za0a14.shoal(amp,amp,x2))
    lbetas.set_ydata(zbeta4.shoal(amp,amp,1.,x2))
    draw()

def up41(val):
    amp=salph41.val
    la0a14s.set_ydata(za0a14.shoal(salph4.val,amp,x2))
    draw()

def up4beta(val):
    amp=salph4beta.val
    lbetas.set_ydata(zbeta4.shoal(salph4.val,salph41.val,amp,x2))
    draw()


salph6.on_changed(up6)
salph4.on_changed(up4)
salph41.on_changed(up41)
salph4beta.on_changed(up4beta)

show()


# figure(2)
# subplot(111)
# xlabel('kH')
# plot(x2,airy.shoal(x2),label='airy')
# plot(x2,zct.shoal(x2),label='zct')
# plot(x2,nwogu2.shoal(x2),label='nw2')
# l6s,= plot(x2,za6.shoal(salph6.val,x2),label='$z=-a h, 6^{th}$ order' )
# l4s,= plot(x2,za4.shoal(salph4.val,x2),label='$z=-a h, 6^{th}$ order' )
# legend()
# show()




######image eps saving#########################
def picture_saving(val=0):
    if (val!=0):
        f1=figure(1)
        xlabel('kH')
        ylabel('Phase velocity $C/\sqrt{gH}$')
        plot(x0,airy.phase(x0),label='airy')
        plot(x0,zct.phase(x0),label='zct')
        plot(x0,z06.phase(x0),label='$z=0, 6^{th}$ order ')
        plot(x0,z04.phase(x0),label='$z=0, 4^{th}$ order ')
        legend(loc=3)
        f1.savefig('fv1test.eps',format='eps')
        close(1)
    
        f2=figure(2)
        xlabel('kH')
        ylabel('Phase velocity $C/\sqrt{gH}$')
        plot(x1,airy.phase(x1),label='airy')
        #plot(x,my3,label='my3')
        plot(x1,zct.phase(x1),label='zct')
        # #plot(x,nwogu1,label='nw1')
        plot(x1,nwogu2.phase(x1),label='nw2')
        #l6,= plot(x1,za6.phase(salph6.val,x1),label='$z=-a h, 6^{th}$ order' )
        #l4, = plot(x1,za4.phase(salph4.val,x1),label='$z=-a h, 4^{th}$ order' )
        legend()
        f2.savefig('fv2test.eps',format='eps')
        close(2)
        
        f3=figure(3)
        xlabel('kH')
        ylabel('Group velocity $C_g/\sqrt{gH}$')
        plot(x0,airy.group(x0),label='airy')
        plot(x0,zct.group(x0),label='zct')
        plot(x0,z06.group(x0),label='$z=0, 6^{th}$ order ')
        plot(x0,z04.group(x0),label='$z=0, 4^{th}$ order ')
        legend(loc=2)
        f3.savefig('fg1test.eps',format='eps')
        close(3)

        f4=figure(4)
        xlabel('kH')
        ylabel('Group velocity $C_g/\sqrt{gH}$')
        plot(x1,airy.group(x1),label='airy')
        plot(x1,zct.group(x1),label='zct')
        plot(x1,nwogu2.group(x1),label='nw2')
        #l6g,= plot(x1,za6.group(salph6.val,x1),label='$z=-a h, 6^{th}$ order' )
        #l4g,= plot(x1,za4.group(salph4.val,x1),label='$z=-a h, 4^{th}$ order' )
        legend(loc=1)
        f4.savefig('fg2test.eps',format='eps')
        close(4)

        f5=figure(5)
        xlabel('kH')
        ylabel('$\\alpha_s$ Shoaling coefficient')
        plot(x01,airy.shoal(x01),label='airy')
        plot(x01,zct.shoal(x01),label='zct')
        plot(x01,z06.shoal(x01),label='z=0,6-order')
        plot(x01,z04.shoal(x01),label='z=0,4-order')
        legend(loc=3)
        f5.savefig('fs1test.eps',format='eps')
        close(5)

        f6=figure(6)
        xlabel('kH')
        plot(x2,airy.shoal(x2),label='airy')
        plot(x2,zct.shoal(x2),label='zct')
        plot(x2,nwogu2.shoal(x2),label='nw1')
        l6s,= plot(x2,za6.shoal(salph6.val,x2),label='$z=-a h, 6^{th}$ order' )
        l4s,= plot(x2,za4.shoal(salph4.val,x2),label='$z=-a h, 4^{th}$ order' )
        la0a14s,= plot(x2,za0a14.shoal(salph4.val,salph41.val,x2),label='$z=-a0 h0-a1 \delta h1, 4^{th}$ order' )
        legend(loc=1)
        f6.savefig('fs22test.eps',format='eps')
        close(6)





picture_saving(save)
print salph6.val
print salph4.val
print salph41.val

# xlabel('kH')
# ylabel('$\\alpha_s$ Shoaling coefficient')
# plot(x0,airy.shoal(x0),label='airy')
# plot(x0,zct.shoal(x0),label='zct')
# plot(x0,z06.shoal(x0),label='z=0,6-order')
# plot(x0,z04.shoal(x0),label='z=0,4-order')
# legend()
# show()

