#!/usr/bin/env python

from pylab import *
from matplotlib.widgets import Slider

#My classes
from airy import *
from zct import *
from lptt16o import *
from lptt14o import *
from lptt26o import *
from lptt24o import *

#ranges 
x0=arange(0.0,30.,0.1)

################## PLOT COMMANDS #######################
beta1=0.0
beta2=1.
beta11=0.0
beta21=1.

airy=AIRY()
zct=ZCT()
lptt16o=LPTt16o(0.58)
lptt14o=LPTt14o(0.59)
lptt26o=LPTt26o(0.59,beta1,beta2)
lptt24o=LPTt24o(0.59,beta11,beta21)

print '''
        *************************************************
        Slider\'s for parameter values\n 
        No dissipative terms\n
        No surface tension terms\n
        **************************************************'''


figure(1)
subplots_adjust(left=0.10,right=0.90,top=0.90, bottom=0.35)
subplot(111)

TITLE='Scaled Phase Velocity Type-I,II'
title(TITLE)

xlabel('$kH$')
ylabel('$C_p/\sqrt{gH}$')

plot(x0,airy.phase(x0),label='airy')
plot(x0,zct.phase(x0),label='zct')
p1,=plot(x0,lptt16o.phase(x0),label='T-I 6-o')
p2,=plot(x0,lptt14o.phase(x0),label='T-I 4-o')
p3,=plot(x0,lptt26o.phase(x0),label='T-II 6-o')
p4,=plot(x0,lptt24o.phase(x0),label='T-II 4-o')
legend(loc=1)


################SLIDER'S DEFINITION ########################
#### axes([left_position(x), bottom_position(y),lenght,height],color...)

color4 = 'lightgoldenrodyellow'
##Slider 1-\alpha lptt16o T-I
s1  = axes([0.15, 0.25, 0.25, 0.03],axisbg=color4)
slider1 = Slider(s1, 'T-I $\\alpha$ $6^{th}$', 0.4, .7, valinit=0.58)
def update_s1(val):
    s1value=slider1.val
    lptt16o=LPTt16o(s1value)
    p1.set_ydata(lptt16o.phase(x0))
    draw()

##Slider 2-\alpha lptt14o T-I
s2  = axes([0.55, 0.25, 0.25, 0.03],axisbg=color4)
slider2 = Slider(s2, '$4^{th}$', 0.4, .7, valinit=0.59)
def update_s2(val):
    s2value=slider2.val
    lptt14o=LPTt14o(s2value)
    p2.set_ydata(lptt14o.phase(x0))
    draw()

##Slider 3-\alpha lptt16o T-II
s3  = axes([0.15, 0.19, 0.25, 0.03],axisbg=color4)
slider3 = Slider(s3, 'T-II $\\alpha 6^{th}$ ', 0.0, 1., valinit=0.59)
def update_s3(val):
    s3value=slider3.val
    lptt26o=LPTt26o(s3value,beta1,beta2)
    p3.set_ydata(lptt26o.phase(x0))
    draw()

s4  = axes([0.55, 0.19, 0.25, 0.03],axisbg=color4)
slider4 = Slider(s4, '$\\beta_1$', 0., .2, valinit=0.)
def update_s4(val):
    s4value=slider4.val
    lptt26o=LPTt26o(slider3.val,s4value,beta2)
    beta1=s4value
    p3.set_ydata(lptt26o.phase(x0))
    draw()

s5  = axes([0.15, 0.13, 0.25, 0.03],axisbg=color4)
slider5 = Slider(s5, '$\\beta_2$', 0.8, 1.0, valinit=1.)
def update_s5(val):
    s5value=slider5.val
    lptt26o=LPTt26o(slider3.val,slider4.val,s5value)
    beta2=s5value
    p3.set_ydata(lptt26o.phase(x0))
    draw()

##Slider 3-\alpha lptt16o T-II
s6  = axes([0.15, 0.07, 0.25, 0.03],axisbg=color4)
slider6 = Slider(s6, 'T-II $\\alpha 4^{th}$ ', 0.0, 1., valinit=0.59)
def update_s6(val):
    s6value=slider6.val
    lptt24o=LPTt26o(s6value,beta11,beta21)
    p4.set_ydata(lptt24o.phase(x0))
    draw()

s7  = axes([0.55, 0.07, 0.25, 0.03],axisbg=color4)
slider7 = Slider(s7, '$\\beta_1$', 0., .2, valinit=0.)
def update_s7(val):
    s7value=slider7.val
    lptt24o=LPTt24o(slider6.val,s7value,beta21)
    beta11=s7value
    p4.set_ydata(lptt24o.phase(x0))
    draw()

s8  = axes([0.15, 0.01, 0.25, 0.03],axisbg=color4)
slider8 = Slider(s8, '$\\beta_2$', 0.8, 1., valinit=1.)
def update_s8(val):
    s8value=slider8.val
    lptt24o=LPTt26o(slider6.val,slider7.val,s8value)
    beta21=s8value
    p4.set_ydata(lptt24o.phase(x0))
    draw()



slider1.on_changed(update_s1)
slider2.on_changed(update_s2)
slider3.on_changed(update_s3)
slider4.on_changed(update_s4)
slider5.on_changed(update_s5)
slider6.on_changed(update_s6)
slider7.on_changed(update_s7)
slider8.on_changed(update_s8)
show()

print 'Slider s1='+str(slider1.val)
print 'Slider s2='+str(slider2.val)
print 'Slider s3='+str(slider3.val)
print 'Slider s4='+str(slider4.val)
print 'Slider s5='+str(slider5.val)
print 'Slider s6='+str(slider6.val)
print 'Slider s7='+str(slider7.val)
print 'Slider s8='+str(slider8.val)
