// {{{ Copyright (C) 2009 Nuno David Lopes.-----------------------------------
//
// e-mail: ndl@ptmat.fc.ul.pt
// http://ptmat.fc.ul.pt/~ndl/
//
// ---------------------------------------------------------------------------
// Licensed under the GNU LGPL Version 3.0 or later.
//
// ---------------------------------------------------------------------------
//  dolfwave.h
//
// First added:  2009-04-14
// Compiled with DOLFIN:0.9.2
// }}}------------------------------------------------------------------------

#include"DolfwaveClass.h"

using namespace dolfin;
using namespace dolfin::dolfwave;


// {{{Constructors--------------------------------------------------------
dolfin::dolfwave::Dolfwave::Dolfwave(dolfin::uint MAXSTEPS ,
                                     double TIMESTEP ,
                                     dolfin::uint WRITEGAP,
                                     std::string MESHNAME,
                                     double PCEPSILON,
                                     std::string FORMNAME,
                                     std::string ALGBACKEND,
                                     std::string PLOTTER,
                                     std::string ETAFILE,
                                     std::string PHIFILE,
                                     std::string PHYPHIFILE,
                                     std::string HORVELFILE)
{
  // {{{Start:Private members to be initialized
  TimerStart();
  dbcapply=false;
  // }}}End:Private members to be initialized

  Time=0.0;  
  // dolfin::uint
  MaxSteps=MAXSTEPS;
  WriteGap=WRITEGAP;
  //

  //Doubles
  TimeStep=TIMESTEP;
  PcEpsilon=PCEPSILON;
  //
  //Strings
  MeshName=MESHNAME;
  FormName=FORMNAME;
  AlgBackEnd=ALGBACKEND;
  Plotter=PLOTTER;
  EtaFile=ETAFILE;
  PhiFile=PHIFILE;
  PhyPhiFile=PHYPHIFILE;
  HorVelFile=HORVELFILE;
  //

  
  etafile= new File(EtaFile,true);
  phifile= new File(PhiFile,true);  
  phyphifile= new File(PhyPhiFile,true); 
  horvelfile= new File(HorVelFile,true); 

  if (AlgBackEnd=="uBLAS")
    {
      dolfin_set("linear algebra backend","uBLAS");  
    }
  else
    {
      message("Backend not implemented yet: Using uBLAS");
      dolfin_set("linear algebra backend","uBLAS");  
    }

  disp();
}

dolfin::dolfwave::Dolfwave::Dolfwave( )
{
  NotImplemented("Use the explicit  constructor",__LINE__,__FILE__);
}

dolfin::dolfwave::Dolfwave::Dolfwave(std::string FILENAME)
{
  std::ifstream  parametersfile (FILENAME.c_str( ));
  NotImplemented("File parameter reading",__LINE__,__FILE__);
  Assert(parametersfile.is_open(),__LINE__,__FILE__);
  /*
    I've a param.c and param.h for this in the old C++ 
    N.-Stokes library 
  */
    disp();
   
}


// }}}------------------------------------------------------------------


// {{{ Info functions --------------------------------------------   
void dolfin::dolfwave::Dolfwave::disp()
{
  
  cout<<"********************************************************"<<endl;
  cout<<"** First Steps towards an Surface Water Waves Library **"<<endl;
  cout<<"********************************************************"<<endl;
  cout<<"**         Some  2nd order models implemented         **"<<endl;
  cout<<"**       Xd3d is preferred for post-processing        **"<<endl;
  cout<<"** http://www.cmap.polytechnique.fr/~jouve/xd3d/      **"<<endl;
  cout<<"********************************************************"<<endl;
  cout<<"********************************************************"<<endl;
  cout<<"*******************Parameters***************************"<<endl;
  cout<<"********************************************MaxSteps  ="<<MaxSteps<<endl;
  cout<<"********************************************TimeStep  ="<<TimeStep<<endl;
  cout<<"********************************************WriteGap  ="<<WriteGap<<endl;
  cout<<"********************************************PcEpsilon ="<<PcEpsilon<<endl;
  cout<<"********************************************FormName  ="<<FormName<<endl;
  cout<<"********************************************MeshName  ="<<MeshName<<endl;
  cout<<"********************************************AlgBackEnd="<<AlgBackEnd<<endl;
  cout<<"********************************************Plotter   ="<<Plotter<<endl;
  cout<<"********************************************EtaFile   ="<<EtaFile<<endl;
  cout<<"********************************************PhiFile   ="<<PhiFile<<endl;
  cout<<"********************************************PhyPhiFile="<<PhyPhiFile<<endl;
  cout<<"********************************************HorVelFile="<<HorVelFile<<endl;
  cout<<"********************************************************"<<endl;
  cout<<"********************************************************"<<endl;
}
// }}}-----------------------------------------------------

