// {{{ Copyright (C) 2009 Nuno David Lopes.-----------------------------------
//
// e-mail: ndl@ptmat.fc.ul.pt
// http://ptmat.fc.ul.pt/~ndl/
//
// ---------------------------------------------------------------------------
// Licensed under the GNU LGPL Version 3.0 or later.
//
// ---------------------------------------------------------------------------
//  PredictorCorrector.h
//
// First added:  2009-04-09
// Compiled with DOLFIN:0.9.1
// }}}------------------------------------------------------------------------


#ifndef __PREDICTORCORRECTOR_H
#define __PREDICTORCORRECTOR_H

#include <dolfin.h>
#include <cstdlib> //For system commands
#include <fstream> //for ofstream File saving


namespace dolfin
{
  namespace dolfwave
  {
    // {{{  Predictor Corrector-Method---------------------------
    class PredCorr
    {
    public:
      PredCorr();
      PredCorr(FunctionSpace& V0,Function**& eta_phi,double eps);
      void pred_assemble(Vector*& pcrk_b,  
                         const dolfin::Form &L0,  
                         const dolfin::Form &L1,  
                         const dolfin::Form &L2,
                         const dolfin::Form &L3,
                         Matrix &M,
                         GenericVector& y,
                         double timestep);
  
      void corr_assemble(Vector*& pcrk_b,  
                         const dolfin::Form &L1,  
                         const dolfin::Form &L2,  
                         const dolfin::Form &L3,
                         const dolfin::Form &L4,
                         Matrix &M,
                         GenericVector& y,
                         double timestep);
  
      void disp(); 

      void update(Function &eta0,Function &eta1,Function &eta2,Function &eta3,Function &eta4,
                  Function &phi0,Function &phi1,Function &phi2,Function &phi3,Function &phi4,
                  Function &eta_phi0,Function &eta_phi1,Function &eta_phi2,Function &eta_phi3,Function &eta_phi4); 
      void update(Function**& eta,Function**& phi,Function**& eta_phi);
      
      void sbfctn_update(Function &eta,Function &phi,Function &eta_phi);

      void pred_solve(Vector*& pcrk_b,
                      const dolfin::Form &L0,
                      const dolfin::Form &L1,
                      const dolfin::Form &L2,
                      const dolfin::Form &L3,
                      Matrix &M,
                      Function &eta_phi3,
                      Function &eta_phi4,
                      Function &eta4,
                      Function &phi4,
                      double  timestep,
                      DirichletBC &dbc,
                      LUSolver &lu,
                      Matrix &Mf,
                      double &t);
      
      void pred_solve(Vector*& pcrk_b,
                      const dolfin::Form &L0,
                      const dolfin::Form &L1,
                      const dolfin::Form &L2,
                      const dolfin::Form &L3,
                      Matrix &M,
                      Function &eta_phi3,
                      Function &eta_phi4,
                      Function &eta4,
                      Function &phi4,
                      double  timestep,
                      LUSolver &lu,
                      double &t);

    
      void corr_solve(Vector*& pcrk_b,
                      const dolfin::Form &L1,
                      const dolfin::Form &L2,
                      const dolfin::Form &L3,
                      const dolfin::Form &L4,
                      Matrix &M,
                      Function &eta_phi3,
                      Function &eta_phi4,
                      Function &eta4,
                      Function &phi4,
                      double  timestep,
                      DirichletBC &dbc,
                      LUSolver &lu,
                      Matrix &Mf,
                      double &t);
 
      void corr_solve(Vector*& pcrk_b,
                      const dolfin::Form &L1,
                      const dolfin::Form &L2,
                      const dolfin::Form &L3,
                      const dolfin::Form &L4,
                      Matrix &M,
                      Function &eta_phi3,
                      Function &eta_phi4,
                      Function &eta4,
                      Function &phi4,
                      double  timestep,
                      LUSolver &lu,
                      double &t);
      
      void solve(Vector*& pcrk_b,
                 dolfin::Form**& L,
                 Matrix &M,
                 Function**& eta_phi,
                 Function**& eta,
                 Function**& phi,
                 double  timestep,
                 DirichletBC &dbc,
                 LUSolver &lu,
                 Matrix &Mf,
                 double &t);
    
      void solve(Vector*& pcrk_b,
                 dolfin::Form**& L,
                 Matrix &M,
                 Function**& eta_phi,
                 Function**& eta,
                 Function**& phi,
                 double  timestep,
                 LUSolver &lu,
                 double &t);
    
    private:
      double p_cts[4];
      double c_cts[4];
      double l2error;
      Function *eta_old;
      Function *phi_old;
      Function *eta_new;
      Function *phi_new;
      Form *ML2N;
      bool multicorrector;
      double PcEpsilon;
    };
    // }}} 
  }
}
#endif 
