// {{{ Copyright (C) 2009 Nuno David Lopes.-----------------------------------
//
// e-mail: ndl@ptmat.fc.ul.pt
// http://ptmat.fc.ul.pt/~ndl/
//
// ---------------------------------------------------------------------------
// Licensed under the GNU LGPL Version 3.0 or later.
//
// ---------------------------------------------------------------------------
//  dolfwave.cpp
//
// First added:  2009-04-09
// Compiled with DOLFIN:0.9.2
// }}}------------------------------------------------------------------------


#include <dolfin.h>

#include "RungeKutta.h"

using namespace dolfin;
using namespace dolfin::dolfwave;

// {{{ Runge-Kutta Method 

void  RungeKutta::rk_assemble(Vector pcrk_b[],  
                              const dolfin::Form &L,  
                              Matrix &M,
                              GenericVector& y,
                              double timestep)
{
  if(rktype==euler)
    {
      assemble(pcrk_b[0],L);//f(y[0])
      M.mult(y,pcrk_b[5]);//pcrk_b[5]=M*y[n]
      pcrk_b[5].axpy(timestep,pcrk_b[0]); //pcrk_b[5]=M*y[n]+h*f(y[n])
    };

  
  if(rktype==exp4)
    {
     //  Vector K[4];
//       Vector B,BAUX;
      
//       ZhaoBilinearForm m(depth);
//       Function eta_phi0(mesh,m);
//       eta_phi0.vector()=v_y;
      
//       Function eta_phi(mesh,m);
//       BAUX=eta_phi.vector();
//       Function eta0(eta_phi0[0]); 
//       Function phi0(eta_phi0[1]);
//       Function etaK(eta_phi[0]); 
//       Function phiK(eta_phi[1]);
      
      
      
//       ZhaoLinearForm L1(depth,eta0,phi0,sponge);
//       ZhaoLinearForm L2(depth,etaK,phiK,sponge);
//       LUSolver solver;
      
//       message("Runge-Kutta Step=1");
      
//       eta0=eta_phi0[0];
//       phi0=eta_phi0[1];
//       assemble(B,L1,mesh);
//       solver.solve(M,K[0],B);
      
//       message("Runge-Kutta Step=2" );
      
//       BAUX=v_y;
//       BAUX.axpy(timestep/2.0,K[0]);
//       etaK=eta_phi[0];
//       phiK=eta_phi[1];
//       assemble(B,L2,mesh);
//       solver.solve(M,K[1],B);
      
//       message("Runge-Kutta Step=3");
      
//       BAUX=v_y;
//       BAUX.axpy(timestep/2.0,K[1]);
//       etaK=eta_phi[0];
//       phiK=eta_phi[1];
//       assemble(B,L2,mesh);
//       solver.solve(M,K[2],B);
      
//       message("Runge-Kutta Step=4");
      
//       BAUX=v_y;
//       BAUX.axpy(timestep,K[2]);
//       etaK=eta_phi[0];
//       phiK=eta_phi[1];
//       assemble(B,L2,mesh);
//       solver.solve(M,K[3],B);
      
//       message("Runge-Kutta Finalizing assemble");
//       b=v_y;
//       b.axpy(timestep/6.0,K[0]);
//       b.axpy(timestep/3.0,K[1]);
//       b.axpy(timestep/3.0,K[2]);
//       b.axpy(timestep/6.0,K[3]);
     };
};    


void RungeKutta::sbfctn_update(Function &eta,Function &phi,Function &eta_phi)
{
  
  
  eta=eta_phi[0];phi=eta_phi[1];//updating function information
};


void RungeKutta::solve(Vector*& pcrk_b,
                       const dolfin::Form &L,
                       Matrix &M,
                       Function &eta_phi0,
                       Function &eta_phi1,
                       Function &eta1,
                       Function &phi1,
                       const double  &timestep,
                       DirichletBC &dbc,
                       LUSolver &lu,
                       Matrix &Mf,
                       double &t)
{
  rk_assemble(pcrk_b,L,M,
              eta_phi0.vector(),
              timestep);
  dbc.apply(Mf,pcrk_b[5]);
  lu.factorized_solve(eta_phi1.vector(),pcrk_b[5]);
  sbfctn_update(eta1,phi1,eta_phi1);//updating subfunction information
  t+=timestep;
  message("Step+=1 \t Simulated time= %.4f",t);
};

void RungeKutta::solve(Vector*& pcrk_b,
                       const dolfin::Form &L,
                       Matrix &M,
                       Function &eta_phi0,
                       Function &eta_phi1,
                       Function &eta1,
                       Function &phi1,
                       const double  &timestep,
                       LUSolver &lu,
                       double &t)
{
  rk_assemble(pcrk_b,L,M,
              eta_phi0.vector(),
              timestep);
  lu.factorized_solve(eta_phi1.vector(),pcrk_b[5]);
  sbfctn_update(eta1,phi1,eta_phi1);//updating subfunction information
  t+=timestep;
  message("Step+=1 \t Simulated time= %.4f",t);
};

// }}}--------------------------------------------------------------------------

// {{{ Generic Solve -----------------------------------------------------------
/* 
with and without Dirichlet boundary conditions
*/
void RungeKutta::solve(Vector*& pcrk_b,
                       dolfin::Form**& L,
                       Matrix &M,
                       Function**&eta_phi,
                       Function**&eta,
                       Function**&phi,
                       const double  &timestep,
                       DirichletBC &dbc,
                       LUSolver &lu,
                       Matrix &Mf,
                       double &t)
{
  solve(pcrk_b,*L[0],M,*eta_phi[0],*eta_phi[1],*eta[1],
        *phi[1],timestep,dbc,lu,Mf,t);
  solve(pcrk_b,*L[1],M,*eta_phi[1],*eta_phi[2],*eta[2],
        *phi[2],timestep,dbc,lu,Mf,t);
  solve(pcrk_b,*L[2],M,*eta_phi[2],*eta_phi[3],*eta[3],
        *phi[3],timestep,dbc,lu,Mf,t);
};

void RungeKutta::solve(Vector*& pcrk_b,
                       dolfin::Form**& L,
                       Matrix &M,
                       Function**&eta_phi,
                       Function**&eta,
                       Function**&phi,
                       const double  &timestep,
                       LUSolver &lu,
                       double &t)
{
  solve(pcrk_b,*L[0],M,*eta_phi[0],*eta_phi[1],*eta[1],
        *phi[1],timestep,lu,t);
  solve(pcrk_b,*L[1],M,*eta_phi[1],*eta_phi[2],*eta[2],
        *phi[2],timestep,lu,t);
  solve(pcrk_b,*L[2],M,*eta_phi[2],*eta_phi[3],*eta[3],
        *phi[3],timestep,lu,t);
};
// }}}--------------------------------------------------------------------------
