// {{{ Copyright (C) 2009 Nuno David Lopes.-----------------------------------
//
// e-mail: ndl@ptmat.fc.ul.pt
// http://ptmat.fc.ul.pt/~ndl/
//
// ---------------------------------------------------------------------------
// Licensed under the GNU LGPL Version 3.0 or later.
//
// ---------------------------------------------------------------------------
//  RungeKutta.h
//
// First added:  2009-04-09
// Compiled with DOLFIN:0.9.2
// }}}------------------------------------------------------------------------

#ifndef __RUNGEKUTTA_H
#define __RUNGEKUTTA_H

#include <dolfin.h>
#include <cstdlib> //For system commands
#include <fstream> //for ofstream File saving

#include "forms/Zhao.h"
#include "forms/L2Norm.h"
#include "forms/L2Proj.h"
#include "forms/SymProj.h"
#include "forms/ZhaoPotentialTrans.h"
#include "forms/LPT2z0.h"

namespace dolfin
{
  namespace dolfwave
  {
    // {{{ Runge-Kutta Method 
    class RungeKutta
    {
    public:
      enum Type {euler,exp4};
      RungeKutta(Type type=euler)
      {
        if(type==euler)
          {
            rktype=euler;
          }
        else if(type==exp4)
          {
            rktype=exp4;
          };
      };
  
      void rk_assemble(Vector pcrk_b[],  
                       const dolfin::Form &L,  
                       Matrix &M,
                       GenericVector& y,
                       double timestep);
      void disp()
      {
        if(rktype==euler)
          {
            cout<<"***************************************"<<endl;
            cout<<"Euler explicit Runge-Kutta method      "<<endl;
            cout<<"***************************************"<<endl;
          }
        else if(rktype==exp4)
          {
            cout<<"***************************************"<<endl;
            cout<<"Explicit Runge-Kutta method 4th order  "<<endl;
            cout<<"***************************************"<<endl;
          }
    
    
      };
   
      void sbfctn_update(Function &eta,Function &phi,Function &eta_phi);

      void solve(Vector*& pcrk_b,
                 const dolfin::Form &L,
                 Matrix &M,
                 Function &eta_phi0,
                 Function &eta_phi1,
                 Function &eta1,
                 Function &phi1,
                 const double  &timestep,
                 DirichletBC &dbc,
                 LUSolver &lu,
                 Matrix &Mf,
                 double &t);

      void solve(Vector*& pcrk_b,
                 const dolfin::Form &L,
                 Matrix &M,
                 Function &eta_phi0,
                 Function &eta_phi1,
                 Function &eta1,
                 Function &phi1,
                 const double  &timestep,
                 LUSolver &lu,
                 double &t);


      void solve(Vector *&,
                 dolfin::Form**& ,
                 Matrix &,
                 Function**&,
                 Function**&,
                 Function**&,
                 const double &,
                 DirichletBC &,
                 LUSolver &,
                 Matrix &,
                 double &);
      
      void solve(Vector *&,
                 dolfin::Form**& ,
                 Matrix &,
                 Function**&,
                 Function**&,
                 Function**&,
                 const double &,
                 LUSolver &,
                 double &);
    
    private:
      Type rktype;
    };
  }
}
#endif
