from ufl import *
from ffc.compiler.uflcompiler import compile as compile_ufl

# Reserved variables for forms
(a, L, M) = (None, None, None)

# Reserved variable for element
element = None

# {{{ Copyright (C) 2009 Nuno David Lopes.-----------------------------------
#
# e-mail: ndl@ptmat.fc.ul.pt
# http://ptmat.fc.ul.pt/~ndl/
#
# ---------------------------------------------------------------------------
# Licensed under the GNU LGPL Version 3.0 or later.
#
# ---------------------------------------------------------------------------
# LPTcdgI.ufl
#
# First added:  2009-07-08
#
# bilinear and linear variational forms for
# Nwogu equations.
#
# The problem is solved in a form M*[eta_t,phi_t]= f(U,eta) 
# Using Runge-Kutta and predictor corrector-schemes
# Using interior penalty method with C/DG method
#              
# Compile this form with FFC: ffc -l dolfin LPTcdgI.ufl
# }}}------------------------------------------------------------------------

######################### Very Experimental #################################

P=FiniteElement("Lagrange",triangle,2) 
Th=P+P

(eta_t,phi_t)=TrialFunctions(Th)#
(p,q)=TestFunctions(Th)


h=Function(P)#For now it should be a constant
eta0=Function(P)
phi0=Function(P)


#Gravity acceleration passed as Constant in *.cpp
#g=Constant(triangle)
g=Function(P)


#Level related parameters
alpha=1-sqrt(6.)/6.
A01=alpha**2/2.-alpha+1./3.
A0=alpha**2/2.-alpha
A5=-alpha**2/6.+alpha/3.-2./15.

# External vector normal
n  = VectorConstant(triangle)

#Penalty parameters
tau = Constant(triangle)
e = Constant(triangle)



## Bilinear Form

## C part
ac0=eta_t*p*dx
ac1=phi_t*q*dx
ac2=-A0*h*h*inner(grad(phi_t),grad(q))*dx
ac3=A5*h*h*h*h*div(grad(phi_t))*div(grad(q))*dx

## DG part
ad0=-A5*h*h*h*h*avg(div(grad(phi_t)))*jump(grad(q),n)*dS
ad1=-A5*h*h*h*h*avg(div(grad(q)))*jump(grad(phi_t),n)*dS

## Penalty terms
ap0=tau('+')/e('+')*inner(jump(grad(eta_t),n), jump(grad(p),n))*dS
ap1=tau('+')/e('+')*inner(jump(grad(phi_t),n), jump(grad(q),n))*dS

## a(h,n,tau,e)
a=ac0+ac1+ac2+ac3+ad0+ad1+ap0+ap1

## Linear Form
## C part
lc0=(h+eta0)*inner(grad(phi0),grad(p))*dx
lc1=-A01*h*h*h*div(grad(phi0))*div(grad(q))*dx
lc2=-g*eta0*q*dx
lc3=-0.5*inner(grad(phi0),grad(phi0))*q*dx

## DG part
ld0=A01*h*h*h*avg(div(grad(phi0)))*jump(grad(p),n)*dS
ld1=A01*h*h*h*avg(div(grad(p)))*jump(grad(phi0),n)*dS

## Penalty terms
lp0=tau('+')/e('+')*inner(jump(grad(eta0),n), jump(grad(p),n))*dS
lp1=tau('+')/e('+')*inner(jump(grad(phi0),n), jump(grad(q),n))*dS

## l(h,eta0,phi,g,n,tau,e)
L=lc0+lc1+lc2+lc3+ld0+ld1+lp0+lp1

compile_ufl([a, L, M, element], "LPTcdgI", {'language': 'dolfin', 'format': 'dolfin', 'form_postfix': True, 'quadrature_order': 'auto', 'precision': '15', 'split_implementation': False, 'cache_dir': None, 'output_dir': '.', 'representation': 'auto', 'optimize': False}, globals())
