from ufl import *
from ffc.compiler.uflcompiler import compile as compile_ufl

# Reserved variables for forms
(a, L, M) = (None, None, None)

# Reserved variable for element
element = None

# {{{ Copyright (C) 2009 Nuno David Lopes.-----------------------------------
#
# e-mail: ndl@ptmat.fc.ul.pt
# http://ptmat.fc.ul.pt/~ndl/
#
# ---------------------------------------------------------------------------
# Licensed under the GNU LGPL Version 3.0 or later.
#
# ---------------------------------------------------------------------------
#  Zhao.ufl
#
# First added:  2008-06-20
#
# bilinear and linear variational forms for
# Nwogu equations.
#
# The problem is solved in a form M*[eta_t,phi_t]= f(U,eta) 
# Using Runge-Kutta and predictor corrector-schemes
#              
# Compile this form with FFC: ffc -l dolfin Zhao.ufl
# }}}------------------------------------------------------------------------


P=FiniteElement("Lagrange",triangle,1) 
Th=P+P

(eta_t,phi_t)=TrialFunctions(Th)#
(p,q)=TestFunctions(Th)


#Depht
h=Function(P)
eta0=Function(P)
phi0=Function(P)


#Gravity acceleration passed as Constant in *.cpp
g=Function(P)

#SpongeLayer
sp_eta=Function(P) #eta0 
sp_lap_eta=Function(P) #Lap eta0 
sp_phi=Function(P) #phi0
sp_lap_phi=Function(P) #Lap phi0

#Source Function
src=Function(P)

#a macros
def t2(h,eta_t,p):
    return 0.5*inner(h*h*grad(eta_t),grad(p))
    
def t3(h,eta_t,p):
    return (1.0/6.0)*inner(grad(eta_t),grad(h*h*p))

def t4(h,eta_t,p):
    return (1.0/15.0)*inner(h*grad(h*eta_t),grad(p))

#Bilinear Variational form for M matrix
a=(eta_t*p+t2(h,eta_t,p)-t3(h,eta_t,p)+t4(h,eta_t,p)+(phi_t*q))*dx 
   
#call with: a(h)

#Linear Variational form for f(U,eta)
def t5(h,eta0,phi0,p):
    return inner(((h+eta0))*grad(phi0),grad(p))

def t6(phi0,q):
    return 0.5*inner(grad(phi0),grad(phi0))*q

def t7(h,eta0,q):
    return g*(1.0/15.0)*inner(h*grad(eta0),grad(h*q))

def speta(sp_eta,sp_lap_eta,eta,p):
    return sp_eta*eta0*p+sp_lap_eta*inner(grad(eta0),grad(p))

def spphi(sp_phi,sp_lap_phi,phi0,q):
    return sp_phi*phi0*q+sp_lap_phi*inner(grad(phi0),grad(q))

#call with L(h, eta0,phi0,sp_eta,sp_lap_eta,sp_phi,sp_lap_phi)
L=(t5(h,eta0,phi0,p)-t6(phi0,q)-g*(eta0*q)-t7(h,eta0,q)\
   -speta(sp_eta,sp_lap_eta,eta0,p)-spphi(sp_phi,sp_lap_phi,phi0,q)+src*p)*dx 

compile_ufl([a, L, M, element], "Zhao", {'language': 'dolfin', 'format': 'dolfin', 'form_postfix': True, 'quadrature_order': 'auto', 'precision': '15', 'split_implementation': False, 'cache_dir': None, 'output_dir': '.', 'representation': 'auto', 'optimize': False}, globals())
