from ufl import *
from ffc.compiler.uflcompiler import compile as compile_ufl

# Reserved variables for forms
(a, L, M) = (None, None, None)

# Reserved variable for element
element = None

# {{{ Copyright (C) 2009 Nuno David Lopes.-----------------------------------
#
# e-mail: ndl@ptmat.fc.ul.pt
# http://ptmat.fc.ul.pt/~ndl/
#
# ---------------------------------------------------------------------------
# Licensed under the GNU LGPL Version 3.0 or later.
#
# ---------------------------------------------------------------------------
#  ZhaoPotential.ufl
#
# First added:  2008-06-20
#
# bilinear and linear variational forms for
# Zhao Teng and Cheng Physical potential equation.
#
#              
# Compile this form with FFC: ffc -l dolfin ZhaoPotentialTrans.ufl
#
# Physical Potential calculation up to order (mu^4)
# Using successive approximation technique
#
# TODO investigate about the coercivity of the original form
# a = (phi0*p)*dx-1./15.*h*inner(grad(Phi),grad(h*p))*dx
#   
# L = p*Phi*dx
#
# Using direct formulations require DG methods
# }}}------------------------------------------------------------------------


P1=FiniteElement("Lagrange",triangle,1)

p = TestFunction(P1)
phi0 = TrialFunction(P1)


h = Function(P1)
Phi = Function(P1)

flux_switch=Function(P1)
n=VectorConstant("triangle")

a = (phi0*p)*dx
   
L = p*Phi*dx+(1./15.)*h*inner(grad(Phi),grad(h*p))*dx\
    -flux_switch*(1./15.)*h*h*inner(grad(Phi),n)*p*ds

compile_ufl([a, L, M, element], "ZhaoPotentialTrans", {'language': 'dolfin', 'format': 'dolfin', 'form_postfix': True, 'quadrature_order': 'auto', 'precision': '15', 'split_implementation': False, 'cache_dir': None, 'output_dir': '.', 'representation': 'auto', 'optimize': False}, globals())
